/**
 * Make directories.
 * This is an apeman task contribution plugin.
 * @module apeman-task-contrib-mkdir
 * @see https://github.com/apeman-repo/apeman
 */

"use strict";

var async = require('async'),
    mkdirp = require('mkdirp'),
    fs = require('fs');

/**
 * Task worker function to make directories.
 * @function mkdir
 * @param {object} context - Task context.
 * @param {object} options - Task options.
 * @param {function} callback - Callback when done.
 */
function mkdir(context, options, callback) {
    var logger = context.logger;

    options = _parseOptions(options);
    _rejectExisting(options.dirnames, function (dirnames) {
        async.eachSeries(dirnames, function (dirname, callback) {
            mkdirp(dirname, function (err) {
                if (!err) {
                    logger.debug('Directory generated: ' + dirname);
                }
                callback(err);
            });
        }, callback);
    });
}

function _parseOptions(options) {
    if (Array.isArray(options)) {
        options = {dirnames: options};
    }
    options.dirnames = [].concat(options.dirnames || []);
    return options;
}

function _rejectExisting(dirnames, callback) {
    async.reject(dirnames, fs.exists, callback);
}


module.exports = mkdir;
